@@ -38,7 +38,7 @@
 #include <unistd.h>
 #include <time.h>
 #include <string.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-exec.h>
 #include <libgnome/gnome-url.h>
@@ -337,6 +337,19 @@ type_from_string (const char *type)
 	return GNOME_DESKTOP_ITEM_TYPE_OTHER;
 }
 
+static void
+init_i18n (void) {
+	static gboolean initialized = FALSE;
+	
+	if (!initialized) {
+		bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
+		bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+#endif
+		initialized = TRUE;
+	}
+}
+
 /**
  * gnome_desktop_item_new:
  *
@@ -349,6 +362,8 @@ gnome_desktop_item_new (void)
 {
 	GnomeDesktopItem *retval;
 
+	init_i18n ();
+
 	retval = g_new0 (GnomeDesktopItem, 1);
 
 	retval->refcount++;
@@ -712,6 +727,77 @@ gnome_desktop_item_new_from_string (const char *uri,
 	return retval;
 }
 
+static char *
+lookup_desktop_file_in_data_dir (const char *desktop_file,
+                                 const char *data_dir)
+{
+	char *path;
+
+	path = g_build_filename (data_dir, "applications", desktop_file, NULL);
+	if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
+		g_free (path);
+		return NULL;
+	}
+	return path;
+}
+
+static char *
+file_from_basename (const char *basename)
+{
+	const char * const *system_data_dirs;
+	const char         *user_data_dir;
+	char               *retval;
+	int                 i;
+
+	user_data_dir = g_get_user_data_dir ();
+	system_data_dirs = g_get_system_data_dirs ();
+
+	if ((retval = lookup_desktop_file_in_data_dir (basename, user_data_dir))) {
+		return retval;
+	}
+	for (i = 0; system_data_dirs[i]; i++) {
+		if ((retval = lookup_desktop_file_in_data_dir (basename, system_data_dirs[i]))) {
+			return retval;
+		}
+	}
+	return NULL;
+}
+
+/**
+ * gnome_desktop_item_new_from_basename:
+ * @basename: The basename of the GnomeDesktopItem to load.
+ * @flags: Flags to influence the loading process
+ *
+ * This function loads 'basename' from a system data directory and 
+ * returns its GnomeDesktopItem. 
+ *
+ * Returns: The newly loaded item.
+ */
+GnomeDesktopItem *
+gnome_desktop_item_new_from_basename (const char *basename,
+                                      GnomeDesktopItemLoadFlags flags,
+                                      GError **error)
+{
+	GnomeDesktopItem *retval;
+	char *file;
+
+	g_return_val_if_fail (basename != NULL, NULL);
+
+	if (!(file = file_from_basename (basename))) {
+		g_set_error (error,
+			     GNOME_DESKTOP_ITEM_ERROR,
+			     GNOME_DESKTOP_ITEM_ERROR_CANNOT_OPEN,
+			     _("Error cannot find file id '%s'"),
+			     basename);
+		return NULL;
+	}
+				   
+	retval = gnome_desktop_item_new_from_file (file, flags, error);
+	g_free (file);
+	
+	return retval;
+}
+
 /**
  * gnome_desktop_item_save:
  * @item: A desktop item
@@ -2506,6 +2592,7 @@ find_kde_directory (void)
 
 /**
  * gnome_desktop_item_find_icon:
+ * @icon_theme: a #GtkIconTheme
  * @icon: icon name, something you'd get out of the Icon key
  * @desired_size: FIXME
  * @flags: FIXME
@@ -2517,13 +2604,17 @@ find_kde_directory (void)
  * Returns: A newly allocated string
  */
 char *
-gnome_desktop_item_find_icon (GnomeIconTheme *icon_theme,
+gnome_desktop_item_find_icon (GtkIconTheme *icon_theme,
 			      const char *icon,
 			      int desired_size,
 			      int flags)
 {
 	char *full = NULL;
 
+	g_return_val_if_fail (icon_theme == NULL ||
+			      GTK_IS_ICON_THEME (icon_theme) ||
+			      GNOME_IS_ICON_THEME (icon_theme), NULL);
+
 	if (icon == NULL || strcmp(icon,"") == 0) {
 		return NULL;
 	} else if (g_path_is_absolute (icon)) {
@@ -2537,11 +2628,10 @@ gnome_desktop_item_find_icon (GnomeIconTheme *icon_theme,
 		char *p;
 
 		if (icon_theme == NULL) {
-			icon_theme = gnome_icon_theme_new ();
+			icon_theme = gtk_icon_theme_new ();
 		} else {
 			g_object_ref (icon_theme);
 		}
-
 		
 		icon_no_extension = g_strdup (icon);
 		p = strrchr (icon_no_extension, '.');
@@ -2552,10 +2642,26 @@ gnome_desktop_item_find_icon (GnomeIconTheme *icon_theme,
 		    *p = 0;
 		}
 
-		full = gnome_icon_theme_lookup_icon (icon_theme,
-						     icon_no_extension,
-						     desired_size,
-						     NULL, NULL);
+		/* For backwards compat we support GnomeIconTheme too */
+		if (GNOME_IS_ICON_THEME (icon_theme)) {
+			full = gnome_icon_theme_lookup_icon (icon_theme,
+							     icon_no_extension,
+							     desired_size,
+							     NULL, NULL);
+		} else {
+			GtkIconInfo *info;
+
+			info = gtk_icon_theme_lookup_icon (icon_theme,
+							   icon_no_extension,
+							   desired_size,
+							   0);
+
+			full = NULL;
+			if (info) {
+				full = g_strdup (gtk_icon_info_get_filename (info));
+				gtk_icon_info_free (info);
+			}
+		}
 		
 		g_object_unref (icon_theme);
 		
@@ -2618,6 +2724,7 @@ gnome_desktop_item_find_icon (GnomeIconTheme *icon_theme,
 
 /**
  * gnome_desktop_item_get_icon:
+ * @icon_theme: a #GtkIconTheme
  * @item: A desktop item
  *
  * Description:  This function goes and looks for the icon file.  If the icon
@@ -2628,7 +2735,7 @@ gnome_desktop_item_find_icon (GnomeIconTheme *icon_theme,
  */
 char *
 gnome_desktop_item_get_icon (const GnomeDesktopItem *item,
-			     GnomeIconTheme *icon_theme)
+			     GtkIconTheme *icon_theme)
 {
 	/* maybe this function should be deprecated in favour of find icon
 	 * -George */
